# Generated by Django 4.2.6 on 2023-10-23 19:44

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("django_napse_core", "0007_historydatapoint_created_at"),
        ("django_napse_auth", "0004_napseapikey_description"),
    ]

    operations = [
        migrations.AlterField(
            model_name="keypermission",
            name="space",
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name="permissions", to="django_napse_core.napsespace"),
        ),
    ]
