# Generated by Django 4.2.6 on 2023-10-17 13:38

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_napse_core", "0004_spacehistory_fleethistory_exchangeaccounthistory"),
    ]

    operations = [
        migrations.AddField(
            model_name="exchangeaccount",
            name="default",
            field=models.BooleanField(default=False),
        ),
    ]
