from django.apps import AppConfig


class DjangoNapseSimulationsConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "django_napse.simulations"
    label = "django_napse_simulations"
