# Generated by Django 4.2.6 on 2023-10-23 19:44

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):
    dependencies = [
        ("django_napse_core", "0006_alter_exchangeaccount_name"),
    ]

    operations = [
        migrations.AddField(
            model_name="historydatapoint",
            name="created_at",
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
    ]
