# Generated by Django 4.2.7 on 2024-02-29 22:33

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("django_napse_auth", "0005_alter_keypermission_space"),
        ("django_napse_simulations", "0001_initial"),
        ("django_napse_core", "0009_rename_exit_base_amount_order_exit_amount_base_and_more"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="NapseSpace",
            new_name="Space",
        ),
    ]
