# Generated by Django 4.2.5 on 2023-10-05 16:16

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("django_napse_core", "0002_history_historydatapoint_historydatapointfield"),
    ]

    operations = [
        migrations.CreateModel(
            name="WalletHistory",
            fields=[
                (
                    "history_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="django_napse_core.history",
                    ),
                ),
                (
                    "owner",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="history",
                        to="django_napse_core.wallet",
                    ),
                ),
            ],
            bases=("django_napse_core.history",),
        ),
    ]
