# Generated by Django 4.2.7 on 2024-01-07 13:23

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_napse_core", "0007_historydatapoint_created_at"),
    ]

    operations = [
        migrations.CreateModel(
            name="BotHistory",
            fields=[
                (
                    "history_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="django_napse_core.history",
                    ),
                ),
                ("owner", models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name="history", to="django_napse_core.bot")),
            ],
            bases=("django_napse_core.history",),
        ),
    ]
